@@ -260,8 +260,7 @@ static void output_immediate(const uint8_t *data, int size,
         data += len;
         size -= len;
 
-        for (; len < 14; len++)
-            avio_w8(out, 0);
+        ffio_fill(out, 0, 14 - len);
 
         (*entries)++;
     }
