@@ -27,7 +27,6 @@
 #include "libavutil/opt.h"
 #include "libavutil/parseutils.h"
 #include "libavutil/pixdesc.h"
-#include "libavutil/avassert.h"
 #include "libavutil/intreadwrite.h"
 
 #define RAW_PACKET_SIZE 1024
@@ -60,7 +59,7 @@ int ff_raw_audio_read_header(AVFormatContext *s)
         return AVERROR(ENOMEM);
     st->codecpar->codec_type = AVMEDIA_TYPE_AUDIO;
     st->codecpar->codec_id = s->iformat->raw_codec_id;
-    st->need_parsing = AVSTREAM_PARSE_FULL_RAW;
+    ffstream(st)->need_parsing = AVSTREAM_PARSE_FULL_RAW;
     st->start_time = 0;
     /* the parameters will be extracted from the compressed bitstream */
 
@@ -71,6 +70,7 @@ int ff_raw_audio_read_header(AVFormatContext *s)
 int ff_raw_video_read_header(AVFormatContext *s)
 {
     AVStream *st;
+    FFStream *sti;
     FFRawVideoDemuxerContext *s1 = s->priv_data;
     int ret = 0;
 
@@ -80,12 +80,13 @@ int ff_raw_video_read_header(AVFormatContext *s)
         ret = AVERROR(ENOMEM);
         goto fail;
     }
+    sti = ffstream(st);
 
     st->codecpar->codec_type = AVMEDIA_TYPE_VIDEO;
     st->codecpar->codec_id = s->iformat->raw_codec_id;
-    st->need_parsing = AVSTREAM_PARSE_FULL_RAW;
+    sti->need_parsing = AVSTREAM_PARSE_FULL_RAW;
 
-    st->internal->avctx->framerate = s1->framerate;
+    sti->avctx->framerate = s1->framerate;
     avpriv_set_pts_info(st, 64, 1, 1200000);
 
 fail:
@@ -103,7 +104,7 @@ int ff_raw_subtitle_read_header(AVFormatContext *s)
     return 0;
 }
 
-int ff_raw_data_read_header(AVFormatContext *s)
+static int raw_data_read_header(AVFormatContext *s)
 {
     AVStream *st = avformat_new_stream(s, NULL);
     if (!st)
@@ -118,30 +119,43 @@ int ff_raw_data_read_header(AVFormatContext *s)
 
 #define OFFSET(x) offsetof(FFRawVideoDemuxerContext, x)
 #define DEC AV_OPT_FLAG_DECODING_PARAM
-const AVOption ff_rawvideo_options[] = {
+static const AVOption rawvideo_options[] = {
     { "framerate", "", OFFSET(framerate), AV_OPT_TYPE_VIDEO_RATE, {.str = "25"}, 0, INT_MAX, DEC},
     { "raw_packet_size", "", OFFSET(raw_packet_size), AV_OPT_TYPE_INT, {.i64 = RAW_PACKET_SIZE }, 1, INT_MAX, DEC},
     { NULL },
 };
 #undef OFFSET
+
+const AVClass ff_rawvideo_demuxer_class = {
+    .class_name = "generic raw video demuxer",
+    .item_name  = av_default_item_name,
+    .option     = rawvideo_options,
+    .version    = LIBAVUTIL_VERSION_INT,
+};
+
 #define OFFSET(x) offsetof(FFRawDemuxerContext, x)
-const AVOption ff_raw_options[] = {
+static const AVOption raw_options[] = {
     { "raw_packet_size", "", OFFSET(raw_packet_size), AV_OPT_TYPE_INT, {.i64 = RAW_PACKET_SIZE }, 1, INT_MAX, DEC},
     { NULL },
 };
 
-#if CONFIG_DATA_DEMUXER
-FF_RAW_DEMUXER_CLASS(raw_data)
+const AVClass ff_raw_demuxer_class = {
+    .class_name = "generic raw demuxer",
+    .item_name  = av_default_item_name,
+    .option     = raw_options,
+    .version    = LIBAVUTIL_VERSION_INT,
+};
 
-AVInputFormat ff_data_demuxer = {
+#if CONFIG_DATA_DEMUXER
+const AVInputFormat ff_data_demuxer = {
     .name           = "data",
     .long_name      = NULL_IF_CONFIG_SMALL("raw data"),
-    .read_header    = ff_raw_data_read_header,
+    .read_header    = raw_data_read_header,
     .read_packet    = ff_raw_read_partial_packet,
     .raw_codec_id   = AV_CODEC_ID_NONE,
     .flags          = AVFMT_NOTIMESTAMPS,
     .priv_data_size = sizeof(FFRawDemuxerContext),\
-    .priv_class     = &raw_data_demuxer_class,\
+    .priv_class     = &ff_raw_demuxer_class,
 };
 #endif
 
