@@ -1,6 +1,7 @@
 /*
  * Westwood Studios VQA Format Demuxer
- * Copyright (c) 2003 The FFmpeg project
+ * Copyright (c) 2003 Mike Melanson <melanson@pcisys.net>
+ * Copyright (c) 2021 Pekka Väänänen <pekka.vaananen@iki.fi>
  *
  * This file is part of FFmpeg.
  *
@@ -30,6 +31,7 @@
 
 #include "libavutil/intreadwrite.h"
 #include "avformat.h"
+#include "avio_internal.h"
 #include "internal.h"
 
 #define FORM_TAG MKBETAG('F', 'O', 'R', 'M')
@@ -40,15 +42,20 @@
 #define SND1_TAG MKBETAG('S', 'N', 'D', '1')
 #define SND2_TAG MKBETAG('S', 'N', 'D', '2')
 #define VQFR_TAG MKBETAG('V', 'Q', 'F', 'R')
+#define VQFL_TAG MKBETAG('V', 'Q', 'F', 'L')
 
 /* don't know what these tags are for, but acknowledge their existence */
 #define CINF_TAG MKBETAG('C', 'I', 'N', 'F')
 #define CINH_TAG MKBETAG('C', 'I', 'N', 'H')
 #define CIND_TAG MKBETAG('C', 'I', 'N', 'D')
+#define LINF_TAG MKBETAG('L', 'I', 'N', 'F')
 #define PINF_TAG MKBETAG('P', 'I', 'N', 'F')
 #define PINH_TAG MKBETAG('P', 'I', 'N', 'H')
 #define PIND_TAG MKBETAG('P', 'I', 'N', 'D')
 #define CMDS_TAG MKBETAG('C', 'M', 'D', 'S')
+#define SN2J_TAG MKBETAG('S', 'N', '2', 'J')
+#define VIEW_TAG MKBETAG('V', 'I', 'E', 'W')
+#define ZBUF_TAG MKBETAG('Z', 'B', 'U', 'F')
 
 #define VQA_HEADER_SIZE 0x2A
 #define VQA_PREAMBLE_SIZE 8
@@ -60,9 +67,11 @@ typedef struct WsVqaDemuxContext {
     int sample_rate;
     int audio_stream_index;
     int video_stream_index;
+    int64_t vqfl_chunk_pos;
+    int vqfl_chunk_size;
 } WsVqaDemuxContext;
 
-static int wsvqa_probe(AVProbeData *p)
+static int wsvqa_probe(const AVProbeData *p)
 {
     /* need 12 bytes to qualify */
     if (p->buf_size < 12)
@@ -85,7 +94,7 @@ static int wsvqa_read_header(AVFormatContext *s)
     uint8_t scratch[VQA_PREAMBLE_SIZE];
     uint32_t chunk_tag;
     uint32_t chunk_size;
-    int fps;
+    int fps, ret;
 
     /* initialize the video decoder stream */
     st = avformat_new_stream(s, NULL);
@@ -101,8 +110,8 @@ static int wsvqa_read_header(AVFormatContext *s)
     avio_seek(pb, 20, SEEK_SET);
 
     /* the VQA header needs to go to the decoder */
-    if (ff_get_extradata(s, st->codecpar, pb, VQA_HEADER_SIZE) < 0)
-        return AVERROR(ENOMEM);
+    if ((ret = ff_get_extradata(s, st->codecpar, pb, VQA_HEADER_SIZE)) < 0)
+        return ret;
     header = st->codecpar->extradata;
     st->codecpar->width = AV_RL16(&header[6]);
     st->codecpar->height = AV_RL16(&header[8]);
@@ -120,6 +129,8 @@ static int wsvqa_read_header(AVFormatContext *s)
     wsvqa->channels     = header[26];
     wsvqa->bps          = header[27];
     wsvqa->audio_stream_index = -1;
+    wsvqa->vqfl_chunk_pos     = 0;
+    wsvqa->vqfl_chunk_size    = 0;
 
     s->ctx_flags |= AVFMTCTX_NOHEADER;
 
@@ -136,11 +147,14 @@ static int wsvqa_read_header(AVFormatContext *s)
         case CINF_TAG:
         case CINH_TAG:
         case CIND_TAG:
+        case LINF_TAG:
         case PINF_TAG:
         case PINH_TAG:
         case PIND_TAG:
         case FINF_TAG:
         case CMDS_TAG:
+        case VIEW_TAG:
+        case ZBUF_TAG:
             break;
 
         default:
@@ -163,18 +177,31 @@ static int wsvqa_read_packet(AVFormatContext *s,
     int ret = -1;
     uint8_t preamble[VQA_PREAMBLE_SIZE];
     uint32_t chunk_type;
-    int chunk_size;
-    unsigned skip_byte;
+    uint32_t chunk_size;
+    int skip_byte;
 
     while (avio_read(pb, preamble, VQA_PREAMBLE_SIZE) == VQA_PREAMBLE_SIZE) {
         chunk_type = AV_RB32(&preamble[0]);
         chunk_size = AV_RB32(&preamble[4]);
 
-        if (chunk_size < 0)
-            return AVERROR_INVALIDDATA;
         skip_byte = chunk_size & 0x01;
 
-        if ((chunk_type == SND0_TAG) || (chunk_type == SND1_TAG) ||
+        if (chunk_type == VQFL_TAG) {
+            /* Each VQFL chunk carries only a codebook update inside which must be applied
+             * before the next VQFR is rendered. That's why we stash the VQFL offset here
+             * so it can be combined with the next VQFR packet. This way each packet
+             * includes a whole frame as expected. */
+            wsvqa->vqfl_chunk_pos = avio_tell(pb);
+            wsvqa->vqfl_chunk_size = (int)(chunk_size);
+            if (wsvqa->vqfl_chunk_size < 0 || wsvqa->vqfl_chunk_size > 3 * (1 << 20))
+                return AVERROR_INVALIDDATA;
+            /* We need a big seekback buffer because there can be SNxx, VIEW and ZBUF
+             * chunks (<512 KiB total) in the stream before we read VQFR (<256 KiB) and
+             * seek back here. */
+            ffio_ensure_seekback(pb, wsvqa->vqfl_chunk_size + (512 + 256) * 1024);
+            avio_skip(pb, chunk_size + skip_byte);
+            continue;
+        } else if ((chunk_type == SND0_TAG) || (chunk_type == SND1_TAG) ||
             (chunk_type == SND2_TAG) || (chunk_type == VQFR_TAG)) {
 
             ret= av_get_packet(pb, pkt, chunk_size);
@@ -216,8 +243,8 @@ static int wsvqa_read_packet(AVFormatContext *s,
                         break;
                     case SND2_TAG:
                         st->codecpar->codec_id = AV_CODEC_ID_ADPCM_IMA_WS;
-                        if (ff_alloc_extradata(st->codecpar, 2))
-                            return AVERROR(ENOMEM);
+                        if ((ret = ff_alloc_extradata(st->codecpar, 2)) < 0)
+                            return ret;
                         AV_WL16(st->codecpar->extradata, wsvqa->version);
                         break;
                     }
@@ -232,11 +259,33 @@ static int wsvqa_read_packet(AVFormatContext *s,
                     break;
                 case SND2_TAG:
                     /* 2 samples/byte, 1 or 2 samples per frame depending on stereo */
-                    pkt->duration = (chunk_size * 2LL) / wsvqa->channels;
+                    pkt->duration = (chunk_size * 2) / wsvqa->channels;
                     break;
                 }
                 break;
             case VQFR_TAG:
+                /* if a new codebook is available inside an earlier a VQFL chunk then
+                 * append it to 'pkt' */
+                if (wsvqa->vqfl_chunk_size > 0) {
+                    int64_t current_pos = pkt->pos;
+
+                    if (avio_seek(pb, wsvqa->vqfl_chunk_pos, SEEK_SET) < 0)
+                        return AVERROR(EIO);
+
+                    /* the decoder expects chunks to be 16-bit aligned */
+                    if (wsvqa->vqfl_chunk_size % 2 == 1)
+                        wsvqa->vqfl_chunk_size++;
+
+                    if (av_append_packet(pb, pkt, wsvqa->vqfl_chunk_size) < 0)
+                        return AVERROR(EIO);
+
+                    if (avio_seek(pb, current_pos, SEEK_SET) < 0)
+                        return AVERROR(EIO);
+
+                    wsvqa->vqfl_chunk_pos = 0;
+                    wsvqa->vqfl_chunk_size = 0;
+                }
+
                 pkt->stream_index = wsvqa->video_stream_index;
                 pkt->duration = 1;
                 break;
@@ -250,6 +299,9 @@ static int wsvqa_read_packet(AVFormatContext *s,
         } else {
             switch(chunk_type){
             case CMDS_TAG:
+            case SN2J_TAG:
+            case VIEW_TAG:
+            case ZBUF_TAG:
                 break;
             default:
                 av_log(s, AV_LOG_INFO, "Skipping unknown chunk %s\n",
@@ -262,7 +314,7 @@ static int wsvqa_read_packet(AVFormatContext *s,
     return ret;
 }
 
-AVInputFormat ff_wsvqa_demuxer = {
+const AVInputFormat ff_wsvqa_demuxer = {
     .name           = "wsvqa",
     .long_name      = NULL_IF_CONFIG_SMALL("Westwood Studios VQA"),
     .priv_data_size = sizeof(WsVqaDemuxContext),
